
--
-- lua-Rotas : <https://fperrad.frama.io/lua-Rotas>
--

local assert = assert
local error = error
local rawset = rawset
local require = require
local setmetatable = setmetatable
local type = type

local _ENV = nil
local m = {}

m.http_methods = {
    'DELETE',
    'GET',
    'HEAD',
    'OPTIONS',
    'PATCH',
    'POST',
    'PUT',
    'TRACE',
}

local function make_key (k)
    if type(k) == 'string' then
        return require('Silva.template')(k)
    end
    assert(type(k) == 'function', 'key not callable')
    return k
end

local function build_single ()
    local cache = {}
    return setmetatable({}, {
        __newindex = function (t, k, v)
            k = make_key(k)
            rawset(t, #t+1, k)
            cache[k] = v
        end,
        __call = function (t, url)
            for i = 1, #t do
                local sme = t[i]
                local capture = sme(url)
                if capture then
                    return cache[sme], capture
                end
            end
        end,
    })
end

local function build_all (all)
    return setmetatable({}, {
        __newindex = function (_, k, v)
            k = make_key(k)
            for i = 1, #all do
                local t = all[i]
                t[k] = v
            end
        end,
        __call = function ()
            error('allowed only for registration')
        end,
    })
end

local function new (t)
    t = t or {}
    local all = {}
    for i = 1, #m.http_methods do
        local meth = m.http_methods[i]
        local tt = build_single()
        t[meth] = tt
        all[#all+1] = tt
    end
    t.ALL = build_all(all)
    return setmetatable(t, {
        __call = function (t2, meth, url)
            if t2[meth] then
                return t2[meth](url)
            end
        end,
    })
end

setmetatable(m, {
    __call = function (_, t) return new(t) end
})

m._NAME = ...
m._VERSION = "0.2.2"
m._DESCRIPTION = "lua-Rotas : a web server router"
m._COPYRIGHT = "Copyright (c) 2018-2023 Francois Perrad"
return m
--
-- This library is licensed under the terms of the MIT/X11 license,
-- like Lua itself.
--
